@@ -1,5 +1,5 @@
 #
-# Copyright 2007 David Snopek <dsnopek@gmail.com>
+# Copyright 2007, 2008 David Snopek <dsnopek@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,15 +23,14 @@ use strict;
 
 sub new
 {
-        my $class = shift;
+	my $class = shift;
 
-        my $self = $class->SUPER::new({
-                package      => 'POE::Component::MessageQueue::Storage::Generic::DBI',
-                options      => \@_,
-        });
+	my $self = $class->SUPER::new({
+		package => 'POE::Component::MessageQueue::Storage::Generic::DBI',
+		options => \@_,
+	});
 
-        bless  $self, $class;
-        return $self;
+	return bless $self, $class;
 }
 
 1;
@@ -99,15 +98,20 @@ L<POE::Component::MessageQueue::Storage::Generic::DBI>).
 
 =head1 SEE ALSO
 
-L<DBI>,
 L<POE::Component::MessageQueue>,
 L<POE::Component::MessageQueue::Storage>,
+L<DBI>
+
+I<Other storage engines:>
+
 L<POE::Component::MessageQueue::Storage::Memory>,
+L<POE::Component::MessageQueue::Storage::BigMemory>,
 L<POE::Component::MessageQueue::Storage::FileSystem>,
 L<POE::Component::MessageQueue::Storage::Generic>,
 L<POE::Component::MessageQueue::Storage::Generic::DBI>,
 L<POE::Component::MessageQueue::Storage::Throttled>,
-L<POE::Component::MessageQueue::Storage::Complex>
+L<POE::Component::MessageQueue::Storage::Complex>,
+L<POE::Component::MessageQueue::Storage::Default>
 
 =cut
 
